@@ -10,14 +10,18 @@ Usage:
   list
   reset
   delete n
+  pause n
+  unpause n
+  opmlexport
+  opmlimport filename
 """
-__version__ = "2.66"
+__version__ = "2.70"
 __author__ = "Lindsey Smith (lindsey@allthingsrss.com)"
 __copyright__ = "(C) 2004 Aaron Swartz. GNU GPL 2 or 3."
 ___contributors__ = ["Dean Jackson", "Brian Lalor", "Joey Hess", 
                      "Matej Cepl", "Martin 'Joey' Schulze", 
                      "Marcel Ackermann (http://www.DreamFlasher.de)", 
-                     "Lindsey Smith", "Aaron Swartz (original author)" ]
+                     "Lindsey Smith (maintainer)", "Erik Hetzner", "Aaron Swartz (original author)" ]
 
 import urllib2
 urllib2.install_opener(urllib2.build_opener())
@@ -73,6 +77,9 @@ AUTHREQUIRED = 0 # if you need to use SMTP AUTH set to 1
 SMTP_USER = 'username'  # for SMTP AUTH, set SMTP username here
 SMTP_PASS = 'password'  # for SMTP AUTH, set SMTP password here
 
+# Connect to the SMTP server using SSL
+SMTP_SSL = 0
+
 # Set this to add a bonus header to all emails (start with '\n').
 BONUS_HEADER = ''
 # Example: BONUS_HEADER = '\nApproved: joe@bob.org'
@@ -80,6 +87,15 @@ BONUS_HEADER = ''
 # Set this to override From addresses. Keys are feed URLs, values are new titles.
 OVERRIDE_FROM = {}
 
+# Set this to override From email addresses. Keys are feed URLs, values are new emails.
+OVERRIDE_EMAIL = {}
+
+# Set this to default From email addresses. Keys are feed URLs, values are new email addresses.
+DEFAULT_EMAIL = {}
+
+# Only use the email from address rather than friendly name plus email address
+NO_FRIENDLY_NAME = 0
+
 # Set this to override the timeout (in seconds) for feed server response
 FEED_TIMEOUT = 60
 
@@ -153,19 +169,23 @@ def send(sender, recipient, subject, body, contenttype, extraheaders=None, smtps
 		
 	fromhdr = formataddr((sender_name, sender_addr))
 	msg['From'] = fromhdr
-		
+
 	msg_as_string = msg.as_string()
 #DEPRECATED 	if QP_REQUIRED:
 #DEPRECATED 		ins, outs = SIO(msg_as_string), SIO()
 #DEPRECATED 		mimify.mimify(ins, outs)
 #DEPRECATED 		msg_as_string = outs.getvalue()
-    		
+
 	if SMTP_SEND:
 		if not smtpserver: 
 			import smtplib
 			
 			try:
-				smtpserver = smtplib.SMTP(SMTP_SERVER)
+				if SMTP_SSL:
+					smtpserver = smtplib.SMTP_SSL()
+				else:
+					smtpserver = smtplib.SMTP()
+				smtpserver.connect(SMTP_SERVER)
 			except KeyboardInterrupt:
 				raise
 			except Exception, e:
@@ -179,7 +199,7 @@ def send(sender, recipient, subject, body, contenttype, extraheaders=None, smtps
 			if AUTHREQUIRED:
 				try:
 					smtpserver.ehlo()
-					smtpserver.starttls()
+					if not SMTP_SSL: smtpserver.starttls()
 					smtpserver.ehlo()
 					smtpserver.login(SMTP_USER, SMTP_PASS)
 				except KeyboardInterrupt:
@@ -283,6 +303,8 @@ h2t.LINKS_EACH_PARAGRAPH = LINKS_EACH_PARAGRAPH
 h2t.BODY_WIDTH = BODY_WIDTH
 html2text = h2t.html2text
 
+from types import *
+
 ### Utility Functions ###
 
 import threading
@@ -371,7 +393,12 @@ def getContent(entry, HTMLOK=0):
 def getID(entry):
 	"""Get best ID from an entry."""
 	if TRUST_GUID:
-		if 'id' in entry and entry.id: return entry.id
+		if 'id' in entry and entry.id: 
+			# Newer versions of feedparser could return a dictionary
+			if type(entry.id) is DictType:
+				return entry.id.values()[0]
+
+			return entry.id
 
 	content = getContent(entry)
 	if content and content != "\n": return hash(unu(content)).hexdigest()
@@ -381,6 +408,8 @@ def getID(entry):
 def getName(r, entry):
 	"""Get the best name."""
 
+	if NO_FRIENDLY_NAME: return ''
+
 	feed = r.feed
 	if hasattr(r, "url") and r.url in OVERRIDE_FROM.keys():
 		return OVERRIDE_FROM[r.url]
@@ -403,26 +432,38 @@ def getName(r, entry):
 	
 	return name
 
-def getEmail(feed, entry):
-	"""Get the best email_address."""
-
+def validateEmail(email, planb):
+	"""Do a basic quality check on email address, but return planb if email doesn't appear to be well-formed"""
+	email_parts = email.split('@')
+	if len(email_parts) != 2:
+		return planb
+	return email
+	
+def getEmail(r, entry):
+	"""Get the best email_address. If the best guess isn't well-formed (something@somthing.com), use DEFAULT_FROM instead"""
+	
+	feed = r.feed
+		
 	if FORCE_FROM: return DEFAULT_FROM
 	
+	if hasattr(r, "url") and r.url in OVERRIDE_EMAIL.keys():
+		return validateEmail(OVERRIDE_EMAIL[r.url], DEFAULT_FROM)
+	
 	if 'email' in entry.get('author_detail', []):
-		return entry.author_detail.email
+		return validateEmail(entry.author_detail.email, DEFAULT_FROM)
 	
 	if 'email' in feed.get('author_detail', []):
-		return feed.author_detail.email
+		return validateEmail(feed.author_detail.email, DEFAULT_FROM)
 		
-	#TODO: contributors
-	
 	if USE_PUBLISHER_EMAIL:
 		if 'email' in feed.get('publisher_detail', []):
-			return feed.publisher_detail.email
+			return validateEmail(feed.publisher_detail.email, DEFAULT_FROM)
 		
 		if feed.get("errorreportsto", ''):
-			return feed.errorreportsto
+			return validateEmail(feed.errorreportsto, DEFAULT_FROM)
 			
+	if hasattr(r, "url") and r.url in DEFAULT_EMAIL.keys():
+		return DEFAULT_EMAIL[r.url]
 	return DEFAULT_FROM
 
 ### Simple Database of Feeds ###
@@ -430,6 +471,7 @@ def getEmail(feed, entry):
 class Feed:
 	def __init__(self, url, to):
 		self.url, self.etag, self.modified, self.seen = url, None, None, {}
+		self.active = True
 		self.to = to		
 
 def load(lock=1):
@@ -454,14 +496,22 @@ def load(lock=1):
 		feeds = pickle.load(feedfileObject)
 		if unix: 
 			fcntl.flock(feedfileObject.fileno(), locktype)
-
+	if feeds: 
+		for feed in feeds[1:]:
+			if not hasattr(feed, 'active'): 
+				feed.active = True
+		
 	return feeds, feedfileObject
 
 def unlock(feeds, feedfileObject):
 	if not unix: 
 		pickle.dump(feeds, open(feedfile, 'w'))
 	else:	
-		pickle.dump(feeds, open(feedfile+'.tmp', 'w'))
+		fd = open(feedfile+'.tmp', 'w')
+		pickle.dump(feeds, fd)
+		fd.flush()
+		os.fsync(fd.fileno())
+		fd.close()
 		os.rename(feedfile+'.tmp', feedfile)
 		fcntl.flock(feedfileObject.fileno(), fcntl.LOCK_UN)
 
@@ -506,6 +556,8 @@ def run(num=None):
 		for f in ifeeds:
 			try: 
 				feednum += 1
+				if not f.active: continue
+				
 				if VERBOSE: print >>warn, 'I: Processing [%d] "%s"' % (feednum, f.url)
 				r = {}
 				try:
@@ -589,13 +641,17 @@ def run(num=None):
 					# Instead of letting these run wild, we put them in context
 					# by associating them with the actual ID (if it exists).
 					
-					frameid = entry.get('id', id)
+					frameid = entry.get('id')
+					if not(frameid): frameid = id
+					if type(frameid) is DictType:
+						frameid = frameid.values()[0]
 					
 					# If this item's ID is in our database
 					# then it's already been sent
 					# and we don't need to do anything more.
 					
-					if f.seen.has_key(frameid) and f.seen[frameid] == id: continue
+					if frameid in f.seen:
+						if f.seen[frameid] == id: continue
 
 					if not (f.to or default_to):
 						print "No default email address defined. Please run 'r2e email emailaddress'"
@@ -620,15 +676,27 @@ def run(num=None):
 						
 					link = entry.get('link', "")
 					
-					from_addr = getEmail(r.feed, entry)
+					from_addr = getEmail(r, entry)
 					
-					name = getName(r, entry)
-					fromhdr = '"'+ name + '" <' + from_addr + ">"
+					name = h2t.unescape(getName(r, entry))
+					fromhdr = formataddr((name, from_addr,))
 					tohdr = (f.to or default_to)
 					subjecthdr = title
 					datehdr = time.strftime("%a, %d %b %Y %H:%M:%S -0000", datetime)
 					useragenthdr = "rss2email"
-					extraheaders = {'Date': datehdr, 'User-Agent': useragenthdr, 'X-RSS-Feed': f.url, 'X-RSS-ID': id}
+					
+					# Add post tags, if available
+					tagline = ""
+					if 'tags' in entry:
+						tags = entry.get('tags')
+						taglist = []
+						if tags:
+							for tag in tags:
+								taglist.append(tag['term'])
+						if taglist:
+							tagline = ",".join(taglist)
+					
+					extraheaders = {'Date': datehdr, 'User-Agent': useragenthdr, 'X-RSS-Feed': f.url, 'X-RSS-ID': id, 'X-RSS-URL': link, 'X-RSS-TAGS' : tagline}
 					if BONUS_HEADER != '':
 						for hdr in BONUS_HEADER.strip().splitlines():
 							pos = hdr.strip().find(':')
@@ -648,7 +716,7 @@ def run(num=None):
 						content += '<div id="entry">\n'
 						content += '<h1'
 						content += ' class="header"'
-						content += '><a href="'+link+'">'+subjecthdr+'</a></h1>\n\n'
+						content += '><a href="'+link+'">'+subjecthdr+'</a></h1>\n'
 						if ishtml(entrycontent):
 							body = entrycontent[1].strip()
 						else:
@@ -662,6 +730,15 @@ def run(num=None):
 									content += ('<br/>Enclosure: <a href="'+enclosure.url+'">'+enclosure.url+"</a>\n")
 								if (hasattr(enclosure, 'src') and enclosure.src != ""):
 									content += ('<br/>Enclosure: <a href="'+enclosure.src+'">'+enclosure.src+'</a><br/><img src="'+enclosure.src+'"\n')
+						if 'links' in entry:
+							for extralink in entry.links:
+								if ('rel' in extralink) and extralink['rel'] == u'via':
+									extraurl = extralink['href']
+									extraurl = extraurl.replace('http://www.google.com/reader/public/atom/', 'http://www.google.com/reader/view/')
+									viatitle = extraurl
+									if ('title' in extralink):
+									    viatitle = extralink['title']
+									content += '<br/>Via: <a href="'+extraurl+'">'+viatitle+'</a>\n'
 						content += '</p></div>\n'
 						content += "\n\n</body></html>"
 					else:	
@@ -677,7 +754,11 @@ def run(num=None):
 								for enclosure in entry.enclosures:
 									if enclosure.url != "":
 										content += ('Enclosure: <a href="'+enclosure.url+'">'+enclosure.url+"</a><br/>\n")
-							
+							if 'links' in entry:
+								for extralink in entry.links:
+									if ('rel' in extralink) and extralink['rel'] == u'via':
+										content += 'Via: <a href="'+extralink['href']+'">'+extralink['title']+'</a><br/>\n'
+                                                                
 							content += ("\n</body></html>")
 						else:
 							content = entrycontent.strip() + "\n\nURL: "+link
@@ -685,6 +766,10 @@ def run(num=None):
 								for enclosure in entry.enclosures:
 									if enclosure.url != "":
 										content += ('\nEnclosure: ' + enclosure.url + "\n")
+							if 'links' in entry:
+								for extralink in entry.links:
+									if ('rel' in extralink) and extralink['rel'] == u'via':
+										content += '<a href="'+extralink['href']+'">Via: '+extralink['title']+'</a>\n'
 
 					smtpserver = send(fromhdr, tohdr, subjecthdr, content, contenttype, extraheaders, smtpserver)
 			
@@ -720,11 +805,53 @@ def list():
 		print "default email:", default_to
 	else: ifeeds = feeds; i = 0
 	for f in ifeeds:
-		print `i`+':', f.url, '('+(f.to or ('default: '+default_to))+')'
+		active = ('[ ]', '[*]')[f.active]
+		print `i`+':',active, f.url, '('+(f.to or ('default: '+default_to))+')'
 		if not (f.to or default_to):
 			print "   W: Please define a default address with 'r2e email emailaddress'"
 		i+= 1
 
+def opmlexport():
+	import xml.sax.saxutils
+	feeds, feedfileObject = load(lock=0)
+	
+	if feeds:
+		print '<?xml version="1.0" encoding="UTF-8"?>\n<opml version="1.0">\n<head>\n<title>rss2email OPML export</title>\n</head>\n<body>'
+		for f in feeds[1:]:
+			url = xml.sax.saxutils.escape(f.url)
+			print '<outline type="rss" text="%s" xmlUrl="%s"/>' % (url, url)
+		print '</body>\n</opml>'
+
+def opmlimport(importfile):
+	importfileObject = None
+	print 'Importing feeds from', importfile
+	if not os.path.exists(importfile):
+		print 'OPML import file "%s" does not exist.' % feedfile
+	try:
+		importfileObject = open(importfile, 'r')
+	except IOError, e:
+		print "OPML import file could not be opened: %s" % e
+		sys.exit(1)
+	try:
+		import xml.dom.minidom
+		dom = xml.dom.minidom.parse(importfileObject)
+		newfeeds = dom.getElementsByTagName('outline')
+	except:
+		print 'E: Unable to parse OPML file'
+		sys.exit(1)
+
+	feeds, feedfileObject = load(lock=1)
+	
+	import xml.sax.saxutils
+	
+	for f in newfeeds:
+		if f.hasAttribute('xmlUrl'):
+			feedurl = f.getAttribute('xmlUrl')
+			print 'Adding %s' % xml.sax.saxutils.unescape(feedurl)
+			feeds.append(Feed(feedurl, None))
+			
+	unlock(feeds, feedfileObject)
+
 def delete(n):
 	feeds, feedfileObject = load()
 	if (n == 0) and (feeds and isstr(feeds[0])):
@@ -738,6 +865,18 @@ def delete(n):
 			print >>warn, "W: feed IDs have changed, list before deleting again"
 	unlock(feeds, feedfileObject)
 	
+def toggleactive(n, active):
+	feeds, feedfileObject = load()
+	if (n == 0) and (feeds and isstr(feeds[0])):
+		print >>warn, "W: ID has to be equal to or higher than 1"
+	elif n >= len(feeds):
+		print >>warn, "W: no such feed"
+	else:
+		action = ('Pausing', 'Unpausing')[active]
+		print >>warn, "%s feed %s" % (action, feeds[n].url)
+		feeds[n].active = active
+	unlock(feeds, feedfileObject)
+	
 def reset():
 	feeds, feedfileObject = load()
 	if feeds and isstr(feeds[0]):
@@ -796,8 +935,24 @@ if __name__ == '__main__':
 			else:
 				raise InputError, "Action '%s' requires a number as its argument" % action
 
+		elif action in ("pause", "unpause"):
+			if not args:
+				raise InputError, "Action '%s' requires an argument" % action
+			elif args[0].isdigit():
+				active = (action == "unpause")
+				toggleactive(int(args[0]), active)
+			else:
+				raise InputError, "Action '%s' requires a number as its argument" % action
+
 		elif action == "reset": reset()
 
+		elif action == "opmlexport": opmlexport()
+
+		elif action == "opmlimport": 
+			if not args:
+				raise InputError, "OPML import '%s' requires a filename argument" % action
+			opmlimport(args[0])
+
 		else:
 			raise InputError, "Invalid action"
 		
